@@ -1,3 +1,19 @@
+/*
+   Copyright The containerd Authors.
+
+   Licensed under the Apache License, Version 2.0 (the "License");
+   you may not use this file except in compliance with the License.
+   You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
+*/
+
 package containerd
 
 import (
@@ -29,7 +45,7 @@ func TestImageIsUnpacked(t *testing.T) {
 	}
 
 	// By default pull does not unpack an image
-	image, err := client.Pull(ctx, imageName)
+	image, err := client.Pull(ctx, imageName, WithPlatform("linux/amd64"))
 	if err != nil {
 		t.Fatal(err)
 	}
